// libs
var demoData = require('../lib/demo');
var pdfCreator = require('../lib/pdfCreator');
var pdfPrinter = require('../lib/pdfPrinter');
var BigDecimal = require('big-decimal');

// properties
var properties = require('../lib/configurationInterpreter');

var log;

var sales;

/**
 * merges all hierarchial outline article that matches "x.y" to their
 * corresponding master Product "x"
 * 
 * @param sales
 * @param i
 * @param pushFn
 * @param taggedSales
 * @returns
 */
function mergeSubProductsOf(sales, i, pushFn, taggedSales) {
	log.info('merging ' + sales[i].description + ' with HierachicalOutline ' + sales[i].hierarchicalOutline);

	log.debug('SubArtikel - Master: ' + sales[i].description);
	// add prices of subarticles to their master
	for (var k = 0; k < sales.length; k++) {
		log.debug('checking if ' + JSON.stringify(sales[k].description) + ' (' + sales[k].hierarchicalOutline + ')' + ' is subArticle of ' + JSON.stringify(sales[i].description) + ' (' + sales[i].hierarchicalOutline + ')');
		if (sales.hasOwnProperty(k) && sales[k].hierarchicalOutline.indexOf('.') > -1 && sales[k].hierarchicalOutline.substring(0, sales[k].hierarchicalOutline.indexOf('.')) === sales[i].hierarchicalOutline) {
			log.info(sales[k].description + ' (' + sales[k].hierarchicalOutline + ')' + ' is subarticle of ' + sales[i].description + ' (' + sales[i].hierarchicalOutline + ')');

			var subPrice = sales[k].price * 1000;
			var subQuantity = sales[k].quantity;
			var masterQuantity = sales[i].quantity;

			masterQuantity = (subQuantity / masterQuantity);

			var masterPrice = subPrice * masterQuantity;

			var price = (parseFloat(sales[i].price) * 1000) + masterPrice;

			price = price / 1000;

			sales[i].price = price.toFixed(2);
		}
	}
	pushFn(taggedSales, sales[i], i);
}

/**
 * pushes all label-tagged sales multiplyed by each quantity property to sale
 * list
 * 
 * @param taggedSales
 * @param sale
 * @param id
 */
function pushTaggedSaleEachQuantity(taggedSales, sale, id) {
	// one label for each sale
	for (var i = 0; i < sale.product.tags.length; i++) {
		if (sale.product.tags[i] === 'label') {
			// one label for each quantity
			for (var q = 0; sale.quantity > q; q++) {
				taggedSales.push(sale);
				if (sale.product.hasOwnProperty('texts') && //
					sale.product.texts.hasOwnProperty('warrantyText')) {
					taggedSales[taggedSales.length - 1].bestBeforeDate = sale.product.texts.warrantyText;
				}
			}
		}
	}
}

/**
 * pushes all label-tagged sales where price is accumulated over quantity
 * 
 * @param taggedSales
 * @param sale
 * @param id
 */
function pushTaggedSaleAcumulatePrice(taggedSales, sale, id) {
	// one label for each sale
	for (var i = 0; i < sale.product.tags.length; i++) {
		if (sale.product.tags[i] === 'label') {
			// acumulate prices
			taggedSales.push(sale);
			sale.price = sale.price * sale.quantity;
			if (sale.product.hasOwnProperty('texts') && //
				sale.product.texts.hasOwnProperty('warrantyText')) {
				taggedSales[taggedSales.length - 1].bestBeforeDate = sale.product.texts.warrantyText;
			}
		}
	}
}

/**
 * simply pushes sale to printable list
 * 
 * @param salesToPrint
 * @param sale
 * @returns
 */
function pushPrintableSale(salesToPrint, sale) {
	for (var s = 0; s < parseInt(sale.quantity); s++) {
		salesToPrint.push(sale);
	}
}

/**
 * pushes every sale to selection list
 * 
 * @param sales
 * @param sale
 * @param id
 */
function pushSale(sales, sale, id) {
	sales.push({
		'id': id,
		'name': sale.description,
		'ean': sale.recognitionCode,
		'amount': sale.quantity,
		'bestBeforeDate': null
	});
	if (sale.product.hasOwnProperty('texts') && sale.product.texts.hasOwnProperty('warrantyText')) {
		sales[sales.length - 1].bestBeforeDate = sale.product.texts.warrantyText;
	}
}

/**
 * creates list of selectable sales and renders selection page
 * 
 * @param req
 * @param res
 */
function processSales(req, res) {
	if (!sales) {
		res.render('print', {
			title: 'Keine Posten gebucht.',
			products: null
		});
		return;
	}

	var selectiveSales = [];

	log.debug('found ' + sales.length + ' sales');

	for (var i = 0; i < sales.length; i++) {
		if (sales[i].hasOwnProperty('product')) {
			if (sales[i].hasOwnProperty('hierarchicalOutline')) {
				if (sales[i].hierarchicalOutline.indexOf('.') === -1) {
					log.debug('SubArtikel - Master: ' + sales[i].description);
					pushSale(selectiveSales, sales[i], i);
				} else {
					log.debug('SubArtikel - Slave: ' + sales[i].description);
				}
			} else {
				pushSale(selectiveSales, sales[i], i);
			}
		}
	}

	log.info('labeled sales: ' + JSON.stringify(selectiveSales));
	log.debug('tagged ' + selectiveSales.length + ' sales');

	res.render('print', {
		title: 'Bitte Labels wählen:',
		products: selectiveSales
	});
}

/**
 * 
 * @param doc
 * @param props
 * @param res
 */
function printDocument(doc, props, res) {
	pdfPrinter.print(doc, props, function (err, stderr, response) {
		if (err) {
			log.error(err);
			log.error(stderr);
			res.send(err);
			return;
		}
		if (stderr) {
			log.error(stderr);
		}
		log.info('document printed');
		res.send('ok');
	});
}

/**
 * POST API Method displaying the selection page
 */
exports.showSelectionDemoPage = function (req, res) {
	log = req.app.get('log');

	log.info('showSelectionDemoPage was called (DEMO)');
	req.body = demoData;

	sales = req.body.request.receipt.sales;

	processSales(req, res);
};

/**
 * GET API Method displaying the selection page with demo data
 */
exports.showSelectionPage = function (req, res) {
	log = req.app.get('log');

	log.info('showSelectionPage was called (NORMAL)');
	var requestJson = JSON.parse(req.body.request);
	sales = requestJson.receipt.sales;

	processSales(req, res);
};

/**
 * POST API Method printing all sales on receipt without any prompt only prints
 * sales with 'label' tag.
 */
exports.instantPrint = function (req, res) {
	log = req.app.get('log');

	log.info('instantPrint was called');
	log.debug(JSON.stringify(req.body, null, 4));
	var sales = req.body.receipt.sales;

	var taggedSales = [];

	log.debug('found ' + sales.length + ' sales');

	for (var i = 0; i < sales.length; i++) {
		if (sales[i].hasOwnProperty('product') && sales[i].product.hasOwnProperty('tags')) {
			if (sales[i].quantity > 0 && sales[i].hasOwnProperty('hierarchicalOutline')) {
				if (sales[i].hierarchicalOutline.indexOf('.') === -1) {
					mergeSubProductsOf(sales, i, pushTaggedSaleEachQuantity, taggedSales);
				} else {
					log.debug('SubArtikel - Slave: ' + sales[i].description);
				}
			} else {
				pushTaggedSaleEachQuantity(taggedSales, sales[i], i);
			}
		}
	}

	properties.loadConfiguration(function (props) {
		if (!taggedSales || taggedSales.length === 0) {
			log.warn('no sales in request!');
			res.send('ok');
			return;
		}

		log.info('labeled sales: ' + JSON.stringify(taggedSales));
		log.debug('tagged ' + taggedSales.length + ' sales');

		pdfCreator.create(taggedSales, function (doc) {
			log.info('pdf document successfully created');

			log.info('printing document...');
			printDocument(doc, props, res);
		});
	});
};
/**
 * POST API Method same as instant print but accumulating prices with quantity
 * instead of printing number of quantity labels
 */
exports.acumulate = function (req, res) {
	log = req.app.get('log');

	log.info('accumulate was called2');
	var sales = req.body.receipt.sales;

	var taggedSales = [];

	log.debug('found ' + sales.length + ' sales');

	for (var i = 0; i < sales.length; i++) {
		if (sales[i].quantity > 0 && sales[i].hasOwnProperty('product') && sales[i].product.hasOwnProperty('tags')) {
			if (sales[i].hasOwnProperty('hierarchicalOutline')) {
				if (sales[i].hierarchicalOutline.indexOf('.') === -1) {
					mergeSubProductsOf(sales, i, pushTaggedSaleAcumulatePrice, taggedSales);
				} else {
					log.debug('SubArtikel - Slave: ' + sales[i].description);
				}
			} else {
				pushTaggedSaleAcumulatePrice(taggedSales, sales[i], i);
			}
		}
	}

	properties.loadConfiguration(function (props) {
		if (!taggedSales || taggedSales.length === 0) {
			log.warn('no sales in request!');
			res.send('ok');
			return;
		}

		log.info('labeled sales: ' + JSON.stringify(taggedSales));
		log.debug('tagged ' + taggedSales.length + ' sales');

		pdfCreator.create(taggedSales, function (doc) {
			log.info('pdf document successfully created');

			log.info('printing document...');
			printDocument(doc, props, res);
		});
	});
};

/**
 * POST API Method for for selection page
 */
exports.printArticles = function (req, res) {
	log = req.app.get('log');

	log.info('printArticles was called');
	log.debug('Request-Body: ' + JSON.stringify(req.body));

	var body = JSON.stringify(req.body);

	body = JSON.parse(body);

	properties.loadConfiguration(function (props) {

		log.debug('sales: ' + JSON.stringify(sales));

		if (!sales || sales.length === 0) {
			log.warn('no sales in request!');
			res.send('ok');
			return;
		}

		if (!req.body || req.body.length === 0) {
			log.warn('no sales were marked on selection page');
			res.send('ok');
			return;
		}

		var salesToPrint = [];
		for (var checkedArticle in req.body) {
			if (body.hasOwnProperty(checkedArticle)) {
				for (var i = 0; i <= sales.length; i++) {
					if (sales.hasOwnProperty(i) && sales[i].description + i === checkedArticle && sales[i].quantity > 0) {
						log.debug('sale: ' + JSON.stringify(sales[i]));
						mergeSubProductsOf(sales, i, pushPrintableSale, salesToPrint);
					}
				}
			}
		}
		log.info('PrintQueue: ' + JSON.stringify(salesToPrint));

		if (!salesToPrint || salesToPrint.length === 0) {
			log.warn('no sales were marked on selection page');
			res.send('ok');
			return;
		} else {
			pdfCreator.create(salesToPrint, function (doc) {
				log.info('PDF document successfully created');
				log.info('printing document...');
				printDocument(doc, props, res);
			});
		}
	});
};
